﻿Shader "Unlit/HologramShader"
{
    Properties
    {
		[Header(Scanline paramaters)]
		_EmissiveColor("Color", Color) = (1,1,1,1)
        _MainTex ("Scanline Texture", 2D) = "white" {}
		_TilingFactor("Tiling Factor", float) = 1
		_Alpha("Opacity", range(0,1)) = 1
		_ScrollSpeed("Scrolling Speed", range(0,1)) = 0

		[Space]
		[Header(Fresnel paramaters)]
		_FresnelIntensity("Fresnel Intensity", range(0,10)) = 1
		_FresnelFalloff("Fresnel Falloff", range(0.1,10)) = 1		

		[Space]
		[Header(Vertex Offset)]
		[Toggle(_VERTEX_OFFSET)] _VERTEX_OFFSET("Use Vertex Offset", Float) = 0
		_OffsetAmount("Offset Amount", range(0,1)) = 0
		_GlitchAmount("Glitch Amount", range(0,1)) = 0
		_GlitchRangeMax("Glitch Range Max", range(-1,1)) = 0
		_GlitchRangeMin ("Glitch Range Min", range(-1,1)) = 0

    }
    SubShader
    {
        Tags { "RenderType"="Transparent" "Queue"="Transparent"}
        LOD 200

		Blend SrcAlpha OneMinusSrcAlpha

		ZWrite On

        Pass
        {
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag
			#pragma shader_feature _VERTEX_OFFSET

            #include "UnityCG.cginc"

            struct appdata
            {
                float4 vertex : POSITION;
                float2 uv : TEXCOORD0;
				float3 normal : NORMAL;
            };

            struct v2f
            {
                float2 uv : TEXCOORD0;
                float4 vertex : SV_POSITION;
				float3 normal : NORMAL;
				half fresnel : TEXCOORD1;
            };

            sampler2D _MainTex;
            float4 _MainTex_ST;
			float _Alpha;
			half _ScrollSpeed;
			half4 _EmissiveColor;
			fixed _TilingFactor;
			half _FresnelIntensity;
			half _FresnelFalloff;
#if defined(_VERTEX_OFFSET)
			fixed _OffsetAmount;
			fixed _GlitchAmount;
			fixed _GlitchRangeMin;
			fixed _GlitchRangeMax;
#endif
			

			float PulseLine(float y, float distance)
			{				
				float pulse = saturate(smoothstep(0, 2, 1 - length(frac(1-y * distance)) * 5));
				return pulse;
			}

#if _VERTEX_OFFSET
			float glitch(float p) {
				return (frac(sin(dot(p, 12.9898)) * 435.5453)) * _GlitchAmount * _OffsetAmount * 2;
			}
#endif

            v2f vert (appdata v)
            {
				v2f o;
				float4 worldPos = mul(unity_ObjectToWorld, v.vertex);
				float4 screenPos = ComputeScreenPos(worldPos);

				float3 viewDir = normalize((_WorldSpaceCameraPos - worldPos).xyz);
				float3 worldNormal = mul(v.normal, (float3x3)unity_WorldToObject);
				o.normal = normalize(worldNormal);

				o.uv = (screenPos.xy / screenPos.w) * _TilingFactor;
				o.uv.y -= _Time.y * _ScrollSpeed;

				half dotP = saturate(1 - dot(worldNormal, viewDir));
				o.fresnel = pow(dotP * _FresnelIntensity, _FresnelFalloff);
#if _VERTEX_OFFSET
				if (dotP < _GlitchRangeMax & dotP > _GlitchRangeMin) {
					worldPos.xz += float2(glitch(screenPos.x), glitch(screenPos.x + 23.465)) * ceil(-sin( _Time.y + sin(_Time.y * 10)) *.455);
				}

				fixed singlePulse = PulseLine(o.uv.y - _Time.y * _ScrollSpeed, .5);
				worldPos.xyz += v.normal * _OffsetAmount * singlePulse;
#endif

				o.vertex = UnityWorldToClipPos(worldPos);

                return o;
            }			

			fixed4 frag(v2f i) : SV_Target
			{		
				float singleLine = PulseLine(i.uv.y - _Time.y * _ScrollSpeed, .5);

                fixed4 col = _EmissiveColor;
				half scan = tex2D(_MainTex, float2(i.uv.x, i.uv.y*.1)).r;
 
				col.a *= i.fresnel + _Alpha * scan + singleLine;	

                return col;
            }
            ENDCG
        }
    }
}
